@@ -1,4 +1,4 @@
-from distutils.version import LooseVersion
+from pkg_resources import parse_version
 
 import pytest
 from _pytest.doctest import DoctestItem
@@ -9,6 +9,10 @@ try:
     import lz4
 except ImportError:
     lz4 = None
+try:
+    from distributed.utils_test import loop
+except ImportError:
+    loop = None
 
 
 def pytest_collection_modifyitems(config, items):
@@ -21,7 +25,7 @@ def pytest_collection_modifyitems(config, items):
             # numpy changed the str/repr formatting of numpy arrays in 1.14.
             # We want to run doctests only for numpy >= 1.14.
             import numpy as np
-            if LooseVersion(np.__version__) >= LooseVersion('1.14'):
+            if parse_version(np.__version__) >= parse_version('1.14'):
                 skip_doctests = False
         except ImportError:
             pass
