@@ -77,10 +77,11 @@ from . import _base
 from .backend import get_context
 from .backend.compat import queue
 from .backend.compat import wait
+from .backend.compat import set_cause
 from .backend.context import cpu_count
 from .backend.queues import Queue, SimpleQueue, Full
-from .backend.utils import recursive_terminate
-from .cloudpickle_wrapper import _wrap_non_picklable_objects
+from .backend.reduction import set_loky_pickler, get_loky_pickler_name
+from .backend.utils import recursive_terminate, get_exitcodes_terminated_worker
 
 try:
     from concurrent.futures.process import BrokenProcessPool as _BPPException
@@ -218,7 +219,8 @@ class _RemoteTraceback(Exception):
 
 class _ExceptionWithTraceback(BaseException):
 
-    def __init__(self, exc, tb=None):
+    def __init__(self, exc):
+        tb = getattr(exc, "__traceback__", None)
         if tb is None:
             _, _, tb = sys.exc_info()
         tb = traceback.format_exception(type(exc), exc, tb)
@@ -231,7 +233,7 @@ class _ExceptionWithTraceback(BaseException):
 
 
 def _rebuild_exc(exc, tb):
-    exc.__cause__ = _RemoteTraceback(tb)
+    exc = set_cause(exc, _RemoteTraceback(tb))
     return exc
 
 
@@ -262,21 +264,17 @@ class _CallItem(object):
         self.args = args
         self.kwargs = kwargs
 
+        # Store the current loky_pickler so it is correctly set in the worker
+        self.loky_pickler = get_loky_pickler_name()
+
+    def __call__(self):
+        set_loky_pickler(self.loky_pickler)
+        return self.fn(*self.args, **self.kwargs)
+
     def __repr__(self):
         return "CallItem({}, {}, {}, {})".format(
             self.work_id, self.fn, self.args, self.kwargs)
 
-    def __getstate__(self):
-        return (
-            self.work_id,
-            _wrap_non_picklable_objects(self.fn),
-            [_wrap_non_picklable_objects(a) for a in self.args],
-            {k: _wrap_non_picklable_objects(a) for k, a in self.kwargs.items()}
-        )
-
-    def __setstate__(self, state):
-        self.work_id, self.fn, self.args, self.kwargs = state
-
 
 class _SafeQueue(Queue):
     """Safe Queue set exception to the future object linked to a job"""
@@ -299,8 +297,8 @@ class _SafeQueue(Queue):
                     "Could not pickle the task to send it to the workers.")
             tb = traceback.format_exception(
                 type(e), e, getattr(e, "__traceback__", None))
-            raised_error.__cause__ = _RemoteTraceback(
-                '\n"""\n{}"""'.format(''.join(tb)))
+            raised_error = set_cause(raised_error, _RemoteTraceback(
+                                     '\n"""\n{}"""'.format(''.join(tb))))
             work_item = self.pending_work_items.pop(obj.work_id, None)
             self.running_work_items.remove(obj.work_id)
             # work_item can be None if another process terminated. In this
@@ -311,11 +309,11 @@ class _SafeQueue(Queue):
                 del work_item
             self.thread_wakeup.wakeup()
         else:
-            super()._on_queue_feeder_error(e, obj)
+            super(_SafeQueue, self)._on_queue_feeder_error(e, obj)
 
 
 def _get_chunks(chunksize, *iterables):
-    """ Iterates over zip()ed iterables in chunks. """
+    """Iterates over zip()ed iterables in chunks. """
     if sys.version_info < (3, 3):
         it = itertools.izip(*iterables)
     else:
@@ -328,7 +326,7 @@ def _get_chunks(chunksize, *iterables):
 
 
 def _process_chunk(fn, chunk):
-    """ Processes a chunk of an iterable passed to map.
+    """Processes a chunk of an iterable passed to map.
 
     Runs the function passed to map() on a chunk of the
     iterable passed to map.
@@ -345,7 +343,7 @@ def _sendback_result(result_queue, work_id, result=None, exception=None):
         result_queue.put(_ResultItem(work_id, result=result,
                                      exception=exception))
     except BaseException as e:
-        exc = _ExceptionWithTraceback(e, getattr(e, "__traceback__", None))
+        exc = _ExceptionWithTraceback(e)
         result_queue.put(_ResultItem(work_id, exception=exc))
 
 
@@ -402,7 +400,7 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
             else:
                 mp.util.info("Could not acquire processes_management_lock")
                 continue
-        except BaseException as e:
+        except BaseException:
             previous_tb = traceback.format_exc()
             try:
                 result_queue.put(_RemoteTraceback(previous_tb))
@@ -417,9 +415,9 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
             with worker_exit_lock:
                 return
         try:
-            r = call_item.fn(*call_item.args, **call_item.kwargs)
+            r = call_item()
         except BaseException as e:
-            exc = _ExceptionWithTraceback(e, getattr(e, "__traceback__", None))
+            exc = _ExceptionWithTraceback(e)
             result_queue.put(_ResultItem(call_item.work_id, exception=exc))
         else:
             _sendback_result(result_queue, call_item.work_id, result=r)
@@ -613,7 +611,7 @@ def _queue_management_worker(executor_reference,
         # signal send. The wake up signals come either from new tasks being
         # submitted, from the executor being shutdown/gc-ed, or from the
         # shutdown of the python interpreter.
-        worker_sentinels = [p.sentinel for p in processes.values()]
+        worker_sentinels = [p.sentinel for p in list(processes.values())]
         ready = wait(readers + worker_sentinels)
 
         broken = ("A worker process managed by the executor was unexpectedly "
@@ -645,10 +643,18 @@ def _queue_management_worker(executor_reference,
         thread_wakeup.clear()
         if broken is not None:
             msg, cause_tb, exc_type = broken
+            if (issubclass(exc_type, TerminatedWorkerError) and
+                    (sys.platform != "win32")):
+                # In Windows, introspecting terminated workers exitcodes seems
+                # unstable, therefore they are not appended in the exception
+                # message.
+                msg += " The exit codes of the workers are {}".format(
+                    get_exitcodes_terminated_worker(processes))
+
             bpe = exc_type(msg)
             if cause_tb is not None:
-                bpe.__cause__ = _RemoteTraceback(
-                    "\n'''\n{}'''".format(''.join(cause_tb)))
+                bpe = set_cause(bpe, _RemoteTraceback(
+                          "\n'''\n{}'''".format(''.join(cause_tb))))
             # Mark the process pool broken so that submits fail right now.
             executor_flags.flag_as_broken(bpe)
 
@@ -847,7 +853,7 @@ class ProcessPoolExecutor(_base.Executor):
 
     def __init__(self, max_workers=None, job_reducers=None,
                  result_reducers=None, timeout=None, context=None,
-                 initializer=None, initargs=()):
+                 initializer=None, initargs=(), env=None):
         """Initializes a new ProcessPoolExecutor instance.
 
         Args:
@@ -868,6 +874,10 @@ class ProcessPoolExecutor(_base.Executor):
                 object should provide SimpleQueue, Queue and Process.
             initializer: An callable used to initialize worker processes.
             initargs: A tuple of arguments to pass to the initializer.
+            env: A dict of environment variable to overwrite in the child
+                process. The environment variables are set before any module is
+                loaded. Note that this only works with the loky context and it
+                is unreliable under windows with Python < 3.6.
         """
         _check_system_limits()
 
@@ -881,11 +891,12 @@ class ProcessPoolExecutor(_base.Executor):
         if context is None:
             context = get_context()
         self._context = context
+        self._env = env
 
         if initializer is not None and not callable(initializer):
             raise TypeError("initializer must be a callable")
-        self._initializer = _wrap_non_picklable_objects(initializer)
-        self._initargs = [_wrap_non_picklable_objects(a) for a in initargs]
+        self._initializer = initializer
+        self._initargs = initargs
 
         _check_max_depth(self._context)
 
@@ -986,17 +997,17 @@ class ProcessPoolExecutor(_base.Executor):
     def _adjust_process_count(self):
         for _ in range(len(self._processes), self._max_workers):
             worker_exit_lock = self._context.BoundedSemaphore(1)
+            args = (self._call_queue, self._result_queue, self._initializer,
+                    self._initargs, self._processes_management_lock,
+                    self._timeout, worker_exit_lock, _CURRENT_DEPTH + 1)
             worker_exit_lock.acquire()
-            p = self._context.Process(
-                target=_process_worker,
-                args=(self._call_queue,
-                      self._result_queue,
-                      self._initializer,
-                      self._initargs,
-                      self._processes_management_lock,
-                      self._timeout,
-                      worker_exit_lock,
-                      _CURRENT_DEPTH + 1))
+            try:
+                # Try to spawn the process with some environment variable to
+                # overwrite but it only works with the loky context for now.
+                p = self._context.Process(target=_process_worker, args=args,
+                                          env=self._env)
+            except TypeError:
+                p = self._context.Process(target=_process_worker, args=args)
             p._worker_exit_lock = worker_exit_lock
             p.start()
             self._processes[p.pid] = p
