@@ -11,7 +11,7 @@ then
 else
     LOKY=$1
 fi
-pip install $LOKY --target $INSTALL_FOLDER
+pip install --no-cache $LOKY --target $INSTALL_FOLDER
 cp -r $INSTALL_FOLDER/loky .
 rm -rf $INSTALL_FOLDER
 
@@ -21,4 +21,10 @@ rm -rf $INSTALL_FOLDER
 find loky -name "*.py" | xargs sed -i.bak "s/from loky/from joblib.externals.loky/"
 find loky -name "*.bak" | xargs rm
 
+for f in $(git grep -l "cloudpickle" loky); do
+    echo $f;
+    sed -i 's/import cloudpickle/from joblib.externals import cloudpickle/' $f
+    sed -i 's/from cloudpickle import/from joblib.externals.cloudpickle import/' $f
+done
+
 sed -i "s/loky.backend.popen_loky/joblib.externals.loky.backend.popen_loky/" loky/backend/popen_loky_posix.py
