@@ -112,7 +112,10 @@ class NumpyArrayWrapper(object):
         if len(self.shape) == 0:
             count = 1
         else:
-            count = unpickler.np.multiply.reduce(self.shape)
+            # joblib issue #859: we cast the elements of self.shape to int64 to
+            # prevent a potential overflow when computing their product.
+            shape_int64 = [unpickler.np.int64(x) for x in self.shape]
+            count = unpickler.np.multiply.reduce(shape_int64)
         # Now read the actual data.
         if self.dtype.hasobject:
             # The array contained Python objects. We need to unpickle the data.
@@ -435,7 +438,9 @@ def dump(value, filename, compress=0, protocol=None, cache_size=None):
     else:
         compress_level = compress
 
-    if compress_method == 'lz4' and lz4 is None:
+    # LZ4 compression is only supported and installation checked with
+    # python 3+.
+    if compress_method == 'lz4' and lz4 is None and PY3_OR_LATER:
         raise ValueError(LZ4_NOT_INSTALLED_ERROR)
 
     if (compress_level is not None and
@@ -548,6 +553,10 @@ def load(filename, mmap_mode=None):
 
     Read more in the :ref:`User Guide <persistence>`.
 
+    WARNING: joblib.load relies on the pickle module and can therefore
+    execute arbitrary Python code. It should therefore never be used
+    to load files from untrusted sources.
+
     Parameters
     -----------
     filename: str, pathlib.Path, or file object.
