@@ -8,9 +8,14 @@ import os
 import sys
 import gc
 
+from joblib._compat import PY3_OR_LATER
 from joblib._multiprocessing_helpers import mp
 from joblib.testing import SkipTest, skipif
 
+try:
+    import lz4
+except ImportError:
+    lz4 = None
 
 # A decorator to run tests only when numpy is available
 try:
@@ -99,6 +104,13 @@ def teardown_autokill(module_name):
 with_multiprocessing = skipif(
     mp is None, reason='Needs multiprocessing to run.')
 
+
 with_dev_shm = skipif(
     not os.path.exists('/dev/shm'),
-    reason='This test requires the /dev/shm shared memory fs.')
+    reason='This test requires a large /dev/shm shared memory fs.')
+
+with_lz4 = skipif(
+    lz4 is None or not PY3_OR_LATER, reason='Needs lz4 compression to run')
+
+without_lz4 = skipif(
+    lz4 is not None, reason='Needs lz4 not being installed to run')
