@@ -572,30 +572,42 @@ def test_numpy_arrays_use_different_memory(mmap_mode):
 
 @with_numpy
 def test_weak_array_key_map():
+
+    def assert_empty_after_gc_collect(container, retries=3):
+        for i in range(retries):
+            if len(container) == 0:
+                return
+            gc.collect()
+        assert len(container) == 0
+
     a = np.ones(42)
     m = _WeakArrayKeyMap()
-    m.set(a, 42)
-    assert m.get(a) == 42
+    m.set(a, 'a')
+    assert m.get(a) == 'a'
 
     b = a
-    assert m.get(b) == 42
-    m.set(b, -42)
-    assert m.get(a) == -42
+    assert m.get(b) == 'a'
+    m.set(b, 'b')
+    assert m.get(a) == 'b'
 
     del a
     gc.collect()
     assert len(m._data) == 1
-    assert m.get(b) == -42
+    assert m.get(b) == 'b'
 
     del b
-    gc.collect()
-    assert len(m._data) == 0
+    assert_empty_after_gc_collect(m._data)
+
+    c = np.ones(42)
+    m.set(c, 'c')
+    assert len(m._data) == 1
+    assert m.get(c) == 'c'
 
-    m.set(np.ones(42), 42)
     with raises(KeyError):
         m.get(np.ones(42))
-    gc.collect()
-    assert len(m._data) == 0
+
+    del c
+    assert_empty_after_gc_collect(m._data)
 
     # Check that creating and dropping numpy arrays with potentially the same
     # object id will not cause the map to get confused.
@@ -620,3 +632,35 @@ def test_weak_array_key_map_no_pickling():
     m = _WeakArrayKeyMap()
     with raises(pickle.PicklingError):
         pickle.dumps(m)
+
+
+@with_numpy
+@with_multiprocessing
+def test_direct_mmap(tmpdir):
+    testfile = str(tmpdir.join('arr.dat'))
+    a = np.arange(10, dtype='uint8')
+    a.tofile(testfile)
+
+    def _read_array():
+        with open(testfile) as fd:
+            mm = mmap.mmap(fd.fileno(), 0, access=mmap.ACCESS_READ, offset=0)
+        return np.ndarray((10,), dtype=np.uint8, buffer=mm, offset=0)
+
+    def func(x):
+        return x**2
+
+    arr = _read_array()
+
+    # this is expected to work and gives the reference
+    ref = Parallel(n_jobs=2)(delayed(func)(x) for x in [a])
+
+    # now test that it work with the mmap array
+    results = Parallel(n_jobs=2)(delayed(func)(x) for x in [arr])
+    np.testing.assert_array_equal(results, ref)
+
+    # also test with a mmap array read in the subprocess
+    def worker():
+        return _read_array()
+
+    results = Parallel(n_jobs=2)(delayed(worker)() for _ in range(1))
+    np.testing.assert_array_equal(results[0], arr)
