@@ -6,6 +6,7 @@ Test the memory module.
 # Copyright (c) 2009 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
+import gc
 import shutil
 import os
 import os.path
@@ -31,6 +32,9 @@ from joblib.testing import parametrize, raises, warns
 from joblib._compat import PY3_OR_LATER
 from joblib.hashing import hash
 
+if sys.version_info[:2] >= (3, 4):
+    import pathlib
+
 
 ###############################################################################
 # Module-level variables for the tests
@@ -330,6 +334,7 @@ def test_memory_numpy_check_mmap_mode(tmpdir, monkeypatch):
     # is necessary to be able edit the file
     del b
     del c
+    gc.collect()
     corrupt_single_cache_item(memory)
 
     # Make sure that corrupting the file causes recomputation and that
@@ -1043,6 +1048,20 @@ def test_memory_default_store_backend():
     excinfo.match(r"Unknown location*")
 
 
+def test_warning_on_unknown_location_type():
+    class NonSupportedLocationClass:
+        pass
+    unsupported_location = NonSupportedLocationClass()
+
+    with warns(UserWarning) as warninfo:
+        _store_backend_factory("local", location=unsupported_location)
+
+    expected_mesage = ("Instanciating a backend using a "
+                       "NonSupportedLocationClass as a location is not "
+                       "supported by joblib")
+    assert expected_mesage in str(warninfo[0].message)
+
+
 def test_instanciate_incomplete_store_backend():
     # Verify that registering an external incomplete store backend raises an
     # exception when one tries to instanciate it.
@@ -1066,6 +1085,15 @@ def test_dummy_store_backend():
     assert isinstance(backend_obj, DummyStoreBackend)
 
 
+@pytest.mark.skipif(sys.version_info[:2] < (3, 4),
+                    reason="pathlib is available for python versions >= 3.4")
+def test_instanciate_store_backend_with_pathlib_path():
+    # Instanciate a FileSystemStoreBackend using a pathlib.Path object
+    path = pathlib.Path("some_folder")
+    backend_obj = _store_backend_factory("local", path)
+    assert backend_obj.location == "some_folder"
+
+
 def test_filesystem_store_backend_repr(tmpdir):
     # Verify string representation of a filesystem store backend.
 
